==> libs2-fw-perl_1.1.tar.xz: s2fw/lib/S2/FW/Plugin/NFT.pm <==
package S2::FW::Plugin::NFT;
use Mojo::Base 'Mojolicious::Plugin';

use Mojo::JSON qw/decode_json/;
use Storable qw/dclone/;

sub register {
    my ($self, $app) = @_;

    $app->helper(query_nft => sub {
            my ($c) = @_;

            my $rows = dclone $c->config->{rows};

            my $nf = decode_json qx(/usr/sbin/nft -j list table s2fw || echo '{"nftables":[{"map":{"name":"host_mode"}}]}');
            my $host_mode    = (grep { defined $_->{map} and $_->{map}->{name} eq 'host_mode' } @{$nf->{nftables}})[0]->{map}->{elem} // [];

            foreach my $row (@$rows) {
                $row->{current_status} = [];
                my $mode = (grep { $_->[0] eq $row->{ip} } @{$host_mode})[0];
                push @{$row->{current_status}}, 'unknown' unless $mode;
                push @{$row->{current_status}}, 'admin' if exists $mode->[1]->{accept};
                push @{$row->{current_status}}, 'web' if exists $mode->[1]->{jump} and $mode->[1]->{jump}->{target} eq 'web';
                push @{$row->{current_status}}, 'blocked' if exists $mode->[1]->{drop};
            }

            return $rows;
        });

    $app->helper(prepare_nft => sub {
            my ($c, $row, $new_mode) = @_;

            my @cmd;
            if ($new_mode eq 'admin') {
                push @cmd, "delete element s2fw host_mode { $row->{ip} }" unless grep { /unknown/i } @{$row->{current_status}};
                push @cmd, "add element s2fw host_mode { $row->{ip} : accept }";
                push @cmd, "add element s2fw port_to_ip { $row->{port_range}->[0]-$row->{port_range}->[1] : $row->{ip} }";
                push @cmd, "add element s2fw port_to_open { $row->{port_range}->[0]-$row->{port_range}->[1] }";
            }
            elsif ($new_mode eq 'web') {
                push @cmd, "delete element s2fw host_mode { $row->{ip} }" unless grep { /unknown/i } @{$row->{current_status}};
                push @cmd, "add element s2fw host_mode { $row->{ip} : jump web }";
                push @cmd, "delete element s2fw port_to_ip { $row->{port_range}->[0]-$row->{port_range}->[1] }" if grep { /admin/i } @{$row->{current_status}};
                push @cmd, "delete element s2fw port_to_open { $row->{port_range}->[0]-$row->{port_range}->[1] }" if grep { /admin/i } @{$row->{current_status}};
            }
            elsif ($new_mode eq 'blocked') {
                push @cmd, "delete element s2fw host_mode { $row->{ip} }" unless grep { /unknown/i } @{$row->{current_status}};
                push @cmd, "add element s2fw host_mode { $row->{ip} : drop }";
                push @cmd, "delete element s2fw port_to_ip { $row->{port_range}->[0]-$row->{port_range}->[1] }" if grep { /admin/i } @{$row->{current_status}};
                push @cmd, "delete element s2fw port_to_open { $row->{port_range}->[0]-$row->{port_range}->[1] }" if grep { /admin/i } @{$row->{current_status}};
            }
            return @cmd;
        });
}

1;

==> libs2-fw-perl_2.0.tar.xz: s2fw/lib/S2/FW/Plugin/NFT.pm <==
package S2::FW::Plugin::NFT;
use Mojo::Base 'Mojolicious::Plugin', -signatures;

use Mojo::JSON qw/decode_json/;
use Storable   qw/dclone/;

sub register ($self, $app, @params) {
    $app->helper(query_nft => sub ($c) {
            my $rows = dclone $c->config->{rows};

            my $nf        = decode_json qx(/usr/sbin/nft -j list table s2fw || echo '{"nftables":[{"map":{"name":"host_mode"}}]}');
            my $host_mode = (grep { defined $_->{map} and $_->{map}->{name} eq 'host_mode' } @{$nf->{nftables}})[0]->{map}->{elem} // [];

            foreach my $row (@$rows) {
                $row->{current_status} = [];
                my $mode = (grep { $_->[0] eq $row->{ip} } @{$host_mode})[0];
                push @{$row->{current_status}}, 'unknown' unless $mode;
                push @{$row->{current_status}}, 'admin'   if exists $mode->[1]->{accept};
                push @{$row->{current_status}}, 'web'     if exists $mode->[1]->{jump} and $mode->[1]->{jump}->{target} eq 'web';
                push @{$row->{current_status}}, 'blocked' if exists $mode->[1]->{drop};
            }

            return $rows;
    });

    $app->helper(prepare_nft => sub ($c, $row, $new_mode) {
            my @cmd;
            if ($new_mode eq 'admin') {
                push @cmd, "delete element s2fw host_mode { $row->{ip} }" unless grep { /unknown/i } @{$row->{current_status}};
                push @cmd, "add element s2fw host_mode { $row->{ip} : accept }";
                push @cmd, "add element s2fw port_to_ip { $row->{port_range}->[0]-$row->{port_range}->[1] : $row->{ip} }";
                push @cmd, "add element s2fw port_to_open { $row->{port_range}->[0]-$row->{port_range}->[1] }";
            }
            elsif ($new_mode eq 'web') {
                push @cmd, "delete element s2fw host_mode { $row->{ip} }" unless grep { /unknown/i } @{$row->{current_status}};
                push @cmd, "add element s2fw host_mode { $row->{ip} : jump web }";
                push @cmd, "delete element s2fw port_to_ip { $row->{port_range}->[0]-$row->{port_range}->[1] }" if grep { /admin/i } @{$row->{current_status}};
                push @cmd, "delete element s2fw port_to_open { $row->{port_range}->[0]-$row->{port_range}->[1] }" if grep { /admin/i } @{$row->{current_status}};
            }
            elsif ($new_mode eq 'blocked') {
                push @cmd, "delete element s2fw host_mode { $row->{ip} }" unless grep { /unknown/i } @{$row->{current_status}};
                push @cmd, "add element s2fw host_mode { $row->{ip} : drop }";
                push @cmd, "delete element s2fw port_to_ip { $row->{port_range}->[0]-$row->{port_range}->[1] }" if grep { /admin/i } @{$row->{current_status}};
                push @cmd, "delete element s2fw port_to_open { $row->{port_range}->[0]-$row->{port_range}->[1] }" if grep { /admin/i } @{$row->{current_status}};
            }
            return @cmd;
    });
}

1;
